@@ -5,28 +5,15 @@ import pytest
 from pyperclip import copy
 
 from .embrace import registry
-from .exc import EmbraceError
-from .gen import gen_text
-from .loader import load
+from .loader import find_embrace_requester, load
 
 
-# TODO entry, inj here
 def pytest_generate_tests(metafunc: pytest.Metafunc) -> None:
-    registry_ = registry()
-    embracers = [name for name in metafunc.fixturenames if name in registry_]
-
-    if len(embracers) > 1:
-        raise EmbraceError(f"Can't request multiple Embrace fixtures. Got {embracers}")
-
-    if len(embracers) == 0:
+    sut = find_embrace_requester(metafunc=metafunc, registry=registry())
+    if sut is None:
         return
-
-    (sut,) = embracers
-    cls = registry_[sut]
-    # ModuleInfo(case_name, case_cls, metafunc.module)
-    cases = load(cls, metafunc.module)
+    cases = load(sut)
     metafunc.parametrize("case", cases, ids=[str(c) for c in cases])
-
     reload(metafunc.module)  # this guarantees the safety of module scope.
     # by reloading at the end of a run, any future references to the just-tested module
     # will encounter it in its 'fresh' state.
@@ -54,7 +41,7 @@ STOP_LOOP = object()  # for pytest hooks that stop on the first-non-None-result
 
 
 def pytest_runtestloop(session: pytest.Session) -> object:
-    generate_for = session.config.getoption("--embrace")
+    generate_for: str = session.config.getoption("--embrace")
     do_ls = session.config.getoption("--embrace-ls")
 
     if generate_for is None and do_ls is None:
@@ -63,14 +50,15 @@ def pytest_runtestloop(session: pytest.Session) -> object:
     reg = registry()
 
     if generate_for is not None:
-        if generate_for not in reg:
+        case_type = registry().get(generate_for)
+        if case_type is None:
             pytest.exit(
                 f"No such fixture '{generate_for}'."
                 f" Your options are {sorted([*reg])}"
             )
 
         # XXX inj here
-        copypasta = gen_text(generate_for)
+        copypasta = case_type.to_text()
         print(f"\nCopying the following output to your clipboard:\n{copypasta}")
         copy(copypasta)
         return STOP_LOOP
